.mermaid #A{
	fill: #971250 !important;
	stroke: blue !important;
	stroke-width: 1.2 !important;
	opacity: 1 !important;
}

.mermaid #B{
	fill: #136009 !important;
	stroke: blue !important;
	stroke-width: 1.2 !important;
	opacity: 1 !important;
}

.mermaid #C{
	fill: #df8f17 !important;
	stroke: blue !important;
	stroke-width: 1.2 !important;
	opacity: 1 !important;
}